"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var deepMerge = require("deepmerge");
var graphql_1 = require("graphql");
var isMergeableObject = function (target) {
    if (!target) {
        return false;
    }
    if (typeof target !== 'object') {
        return false;
    }
    var stringValue = Object.prototype.toString.call(target);
    if (stringValue === '[object RegExp]') {
        return false;
    }
    if (stringValue === '[object Date]') {
        return false;
    }
    if (target instanceof graphql_1.GraphQLScalarType) {
        return false;
    }
    return true;
};
function mergeResolvers(resolversDefinitions) {
    if (!resolversDefinitions || resolversDefinitions.length === 0) {
        return {};
    }
    if (resolversDefinitions.length === 1) {
        return resolversDefinitions[0];
    }
    var resolversFactories = new Array();
    var resolvers = new Array();
    for (var _i = 0, resolversDefinitions_1 = resolversDefinitions; _i < resolversDefinitions_1.length; _i++) {
        var resolversDefinition = resolversDefinitions_1[_i];
        if (typeof resolversDefinition === 'function') {
            resolversFactories.push(resolversDefinition);
        }
        else if (typeof resolversDefinition === 'object') {
            resolvers.push(resolversDefinition);
        }
    }
    if (resolversFactories.length) {
        return (function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var resultsOfFactories = resolversFactories.map(function (factory) { return factory.apply(void 0, args); });
            return deepMerge.all(resolvers.concat(resultsOfFactories), { isMergeableObject: isMergeableObject });
        });
    }
    else {
        return deepMerge.all(resolvers, { isMergeableObject: isMergeableObject });
    }
}
exports.mergeResolvers = mergeResolvers;
//# sourceMappingURL=merge-resolvers.js.map