"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_1 = require("graphql");
var helpers_1 = require("./helpers");
function fixSchemaAst(schema) {
    return graphql_1.buildASTSchema(graphql_1.parse(graphql_1.printSchema(schema)));
}
function extendDefinition(type) {
    switch (type.astNode.kind) {
        case 'ObjectTypeDefinition':
            return __assign({}, type.astNode, { 
                // add fields from object extension (`extend type Query { newField: String }`)
                fields: type.astNode.fields.concat(type.extensionASTNodes.reduce(function (fields, node) { return fields.concat(node.fields); }, [])) });
        default:
            return type.astNode;
    }
}
function printSchema(schema) {
    var typesMap = schema.getTypeMap();
    var validAstNodes = Object.keys(typesMap).filter(function (key) { return typesMap[key].astNode; });
    if (validAstNodes.length === 0 && Object.keys(typesMap).length > 0) {
        schema = fixSchemaAst(schema);
        typesMap = schema.getTypeMap();
    }
    var schemaDefinition = helpers_1.createSchemaDefinition({
        query: schema.getQueryType(),
        mutation: schema.getMutationType(),
        subscription: schema.getSubscriptionType(),
    });
    var allTypesPrinted = Object.keys(typesMap)
        .map(function (key) { return typesMap[key]; })
        .filter(function (type) {
        var isPredefinedScalar = type instanceof graphql_1.GraphQLScalarType && graphql_1.isSpecifiedScalarType(type);
        var isIntrospection = graphql_1.isIntrospectionType(type);
        return !isPredefinedScalar && !isIntrospection;
    })
        .map(function (type) {
        if (type.astNode) {
            return graphql_1.print(type.extensionASTNodes ? extendDefinition(type) : type.astNode);
        }
        else {
            return graphql_1.printType(type);
        }
    })
        .filter(function (e) { return e; });
    var directivesDeclaration = schema
        .getDirectives()
        .map(function (directive) { return (directive.astNode ? graphql_1.print(directive.astNode) : null); })
        .filter(function (e) { return e; });
    var printedSchema = directivesDeclaration.concat(allTypesPrinted, [schemaDefinition]).join('\n');
    return graphql_1.parse(printedSchema);
}
exports.printSchema = printSchema;
//# sourceMappingURL=print-schema.js.map